"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var EvaluationMetadata_1 = require("./EvaluationMetadata");
var ConfigEvaluation = /** @class */ (function () {
    function ConfigEvaluation(value, rule_id, secondary_exposures, json_value, explicit_parameters, config_delegate) {
        if (secondary_exposures === void 0) { secondary_exposures = []; }
        if (json_value === void 0) { json_value = {}; }
        if (explicit_parameters === void 0) { explicit_parameters = null; }
        if (config_delegate === void 0) { config_delegate = null; }
        this.group_name = null;
        this.value = value;
        this.rule_id = rule_id;
        if (typeof json_value === 'boolean') {
            // handle legacy gate case
            this.json_value = {};
        }
        else {
            this.json_value = json_value;
        }
        this.secondary_exposures = secondary_exposures;
        this.undelegated_secondary_exposures = secondary_exposures;
        this.config_delegate = config_delegate;
        this.explicit_parameters = explicit_parameters;
        this.is_experiment_group = false;
        this.evaluation_details = {
            time: Date.now(),
            reason: EvaluationMetadata_1.EvaluationReason.Uninitialized,
        };
    }
    ConfigEvaluation.prototype.withGroupName = function (name) {
        this.group_name = name;
        return this;
    };
    ConfigEvaluation.prototype.withEvaluationDetails = function (reason, time) {
        this.evaluation_details.reason = reason;
        this.evaluation_details.time = time;
        return this;
    };
    ConfigEvaluation.prototype.setIsExperimentGroup = function (isExperimentGroup) {
        if (isExperimentGroup === void 0) { isExperimentGroup = false; }
        this.is_experiment_group = isExperimentGroup;
    };
    ConfigEvaluation.fromSticky = function (stickyValue) {
        var evaluation = new ConfigEvaluation(stickyValue.value, stickyValue.rule_id, stickyValue.secondary_exposures, stickyValue.json_value, stickyValue.explicit_parameters, stickyValue.config_delegate);
        evaluation.evaluation_details = {
            time: stickyValue.time,
            reason: EvaluationMetadata_1.EvaluationReason.Persisted,
        };
        evaluation.undelegated_secondary_exposures =
            stickyValue.undelegated_secondary_exposures;
        evaluation.is_experiment_group = true;
        return evaluation.withGroupName(stickyValue.group_name);
    };
    ConfigEvaluation.prototype.getJSONValue = function () {
        var _a;
        return {
            value: this.value,
            rule_id: this.rule_id,
            json_value: this.json_value,
            secondary_exposures: this.secondary_exposures,
            explicit_parameters: this.explicit_parameters,
            config_delegate: this.config_delegate,
            undelegated_secondary_exposures: (_a = this.undelegated_secondary_exposures) !== null && _a !== void 0 ? _a : null,
            group_name: this.group_name,
            time: this.evaluation_details.time,
        };
    };
    return ConfigEvaluation;
}());
exports.default = ConfigEvaluation;
