"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigCondition = exports.ConfigRule = exports.ConfigSpec = void 0;
var ConfigSpec = /** @class */ (function () {
    function ConfigSpec(specJSON) {
        var _a;
        this.name = specJSON.name;
        this.type = specJSON.type;
        this.salt = specJSON.salt;
        this.defaultValue = specJSON.defaultValue;
        this.enabled = specJSON.enabled;
        this.idType = specJSON.idType;
        this.rules = this.parseRules(specJSON.rules);
        this.entity = specJSON.entity;
        this.explicitParameters = ((_a = specJSON.explicitParameters) !== null && _a !== void 0 ? _a : null);
        if (specJSON.isActive !== null) {
            this.isActive = specJSON.isActive;
        }
        this.hasSharedParams =
            specJSON.hasSharedParams != null
                ? specJSON.hasSharedParams === true
                : false;
    }
    ConfigSpec.prototype.parseRules = function (rulesJSON) {
        var json = rulesJSON;
        var rules = [];
        for (var i = 0; i < json.length; i++) {
            var rule = new ConfigRule(json[i]);
            rules.push(rule);
        }
        return rules;
    };
    ConfigSpec.prototype.toRecord = function () {
        return {
            name: this.name,
            type: this.type,
            salt: this.salt,
            defaultValue: this.defaultValue,
            enabled: this.enabled,
            idType: this.idType,
            rules: this.rules.map(function (rule) { return rule.toRecord(); }),
            entity: this.entity,
            explicitParameters: this.explicitParameters,
            hasSharedParams: this.hasSharedParams,
            isActive: this.isActive,
        };
    };
    return ConfigSpec;
}());
exports.ConfigSpec = ConfigSpec;
var ConfigRule = /** @class */ (function () {
    function ConfigRule(ruleJSON) {
        var _a, _b;
        this.name = ruleJSON.name;
        this.passPercentage = ruleJSON.passPercentage;
        this.conditions = this.parseConditions(ruleJSON.conditions);
        this.returnValue = ruleJSON.returnValue;
        this.id = ruleJSON.id;
        this.salt = ruleJSON.salt;
        this.idType = ruleJSON.idType;
        this.configDelegate = (_a = ruleJSON.configDelegate) !== null && _a !== void 0 ? _a : null;
        if (ruleJSON.isExperimentGroup !== null) {
            this.isExperimentGroup = ruleJSON.isExperimentGroup;
        }
        this.groupName = (_b = ruleJSON.groupName) !== null && _b !== void 0 ? _b : null;
    }
    ConfigRule.prototype.toRecord = function () {
        return {
            name: this.name,
            passPercentage: this.passPercentage,
            conditions: this.conditions.map(function (condition) { return condition.toRecord(); }),
            returnValue: this.returnValue,
            id: this.id,
            salt: this.salt,
            idType: this.idType,
            configDelegate: this.configDelegate,
            isExperimentGroup: this.isExperimentGroup,
            groupName: this.groupName,
        };
    };
    ConfigRule.prototype.parseConditions = function (conditionsJSON) {
        var json = conditionsJSON;
        var conditions = [];
        json === null || json === void 0 ? void 0 : json.forEach(function (cJSON) {
            var condition = new ConfigCondition(cJSON);
            conditions.push(condition);
        });
        return conditions;
    };
    return ConfigRule;
}());
exports.ConfigRule = ConfigRule;
var ConfigCondition = /** @class */ (function () {
    function ConfigCondition(conditionJSON) {
        var _a;
        this.type = conditionJSON.type;
        this.targetValue = conditionJSON.targetValue;
        this.operator = conditionJSON.operator;
        this.field = conditionJSON.field;
        this.additionalValues =
            (_a = conditionJSON.additionalValues) !== null && _a !== void 0 ? _a : {};
        this.idType = conditionJSON.idType;
    }
    ConfigCondition.prototype.toRecord = function () {
        return {
            type: this.type,
            targetValue: this.targetValue,
            operator: this.operator,
            field: this.field,
            additionalValues: this.additionalValues,
            idType: this.idType,
        };
    };
    return ConfigCondition;
}());
exports.ConfigCondition = ConfigCondition;
