import { UserPersistedValues } from "./utils/StickyValuesStorage";
export declare const DEFAULT_CONFIG_SPEC_API = "https://api.statsigcdn.com/v1/download_config_specs/";
export declare const DEFAULT_EVENT_LOGGING_API = "https://events.statsigapi.net/v1/rgstr";
export declare const INIT_TIMEOUT_DEFAULT_MS = 3000;
export type StatsigEnvironment = {
    tier?: 'production' | 'staging' | 'development' | string;
    [key: string]: string | undefined;
};
export interface UserPersistentStorageInterface {
    delete(key: string, experiment: string): void;
    load(key: string): UserPersistedValues;
    save(key: string, experiment: string, data: string): void;
    loadAsync(key: string): Promise<UserPersistedValues>;
}
type CommonOptions = {
    configSpecAPI?: string;
    eventLoggingAPI?: string;
    disableCurrentPageLogging?: boolean;
    environment?: StatsigEnvironment;
    loggingIntervalMillis?: number;
    loggingBufferMaxSize?: number;
    disableNetworkKeepalive?: boolean;
    overrideStableID?: string;
    localMode?: boolean;
    disableLocalStorage?: boolean;
    ignoreWindowUndefined?: boolean;
    userPersistentStorage?: UserPersistentStorageInterface;
};
export type StatsigOptions = CommonOptions & {
    initTimeoutMs?: number;
};
export type SynchronousStatsigOptions = CommonOptions & {
    initializeValues: Record<string, any>;
};
export default class StatsigSDKOptions {
    readonly configSpecAPI: string;
    readonly eventLoggingAPI: string;
    readonly disableCurrentPageLogging: boolean;
    readonly environment: StatsigEnvironment | null;
    readonly loggingIntervalMillis: number;
    readonly loggingBufferMaxSize: number;
    readonly disableNetworkKeepalive: boolean;
    readonly overrideStableID: string | null;
    readonly localMode: boolean;
    readonly initTimeoutMs: number;
    readonly initializeValues: Record<string, any> | null;
    readonly disableLocalStorage: boolean;
    readonly ignoreWindowUndefined: boolean;
    readonly userPersistentStorage: UserPersistentStorageInterface | null;
    constructor(options?: SynchronousStatsigOptions | StatsigOptions | null);
    private normalizeNumberInput;
}
export {};
